@@ -25,11 +25,6 @@
 #define SETTINGS_SCHEMA "org.mate.SettingsDaemon.plugins.housekeeping"
 #define SETTINGS_IGNORE_PATHS "ignore-paths"
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
-#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
-#endif
-
 enum
 {
         PROP_0,
@@ -220,17 +215,14 @@ msd_ldsm_dialog_init (MsdLdsmDialog *dialog)
 
         /* Create the image */
         image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_DIALOG);
-#if GTK_CHECK_VERSION (3, 0, 0)
         gtk_widget_set_halign (image, GTK_ALIGN_START);
         gtk_widget_set_valign (image, GTK_ALIGN_END);
-#else
-        gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-#endif
 
         /* Create the labels */
         dialog->priv->primary_label = gtk_label_new (NULL);	
         gtk_label_set_line_wrap (GTK_LABEL (dialog->priv->primary_label), TRUE);
         gtk_label_set_single_line_mode (GTK_LABEL (dialog->priv->primary_label), FALSE);
+        gtk_label_set_max_width_chars (GTK_LABEL (dialog->priv->primary_label), 70);
 #if GTK_CHECK_VERSION (3, 16, 0)
         gtk_label_set_xalign (GTK_LABEL (dialog->priv->primary_label), 0.0);
         gtk_label_set_yalign (GTK_LABEL (dialog->priv->primary_label), 0.0);
@@ -241,6 +233,7 @@ msd_ldsm_dialog_init (MsdLdsmDialog *dialog)
         dialog->priv->secondary_label = gtk_label_new (NULL);
         gtk_label_set_line_wrap (GTK_LABEL (dialog->priv->secondary_label), TRUE);
         gtk_label_set_single_line_mode (GTK_LABEL (dialog->priv->secondary_label), FALSE);
+        gtk_label_set_max_width_chars (GTK_LABEL (dialog->priv->secondary_label), 70);
 #if GTK_CHECK_VERSION (3, 16, 0)
         gtk_label_set_xalign (GTK_LABEL (dialog->priv->secondary_label), 0.0);
         gtk_label_set_yalign (GTK_LABEL (dialog->priv->secondary_label), 0.0);
@@ -261,10 +254,10 @@ msd_ldsm_dialog_init (MsdLdsmDialog *dialog)
         /* Now set up the dialog's GtkBox's' */
         gtk_box_set_spacing (GTK_BOX (main_vbox), 14);
 	
-        hbox = gtk_hbox_new (FALSE, 12);
+        hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
         gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
 	
-        text_vbox = gtk_vbox_new (FALSE, 12);
+        text_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
         
         gtk_box_pack_start (GTK_BOX (text_vbox), dialog->priv->primary_label, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX (text_vbox), dialog->priv->secondary_label, TRUE, TRUE, 0);
