@@ -330,7 +330,8 @@ binding_unregister_keys (MsdKeybindingsManager *manager)
 
         if (need_flush)
                 gdk_flush ();
-        gdk_error_trap_pop ();
+
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -517,14 +518,8 @@ keybindings_filter (GdkXEvent             *gdk_xevent,
 
 static void
 bindings_callback (DConfClient           *client,
-        #ifdef HAVE_DCONF_0_13
                    gchar                 *prefix,
                    GStrv                  changes,
-        #else
-                   const gchar           *path,
-                   const gchar * const   *items,
-                   gint                   n_items,
-        #endif
                    gchar                 *tag,
                    MsdKeybindingsManager *manager)
 {
@@ -543,20 +538,34 @@ msd_keybindings_manager_start (MsdKeybindingsManager *manager,
 {
         GdkDisplay  *dpy;
         GdkScreen   *screen;
+        GdkWindow   *window;
         int          screen_num;
         int          i;
+        Display     *xdpy;
+        Window       xwindow;
+        XWindowAttributes atts;
 
         g_debug ("Starting keybindings manager");
         mate_settings_profile_start (NULL);
 
         dpy = gdk_display_get_default ();
+        xdpy = GDK_DISPLAY_XDISPLAY (dpy);
         screen_num = gdk_display_get_n_screens (dpy);
 
         for (i = 0; i < screen_num; i++) {
                 screen = gdk_display_get_screen (dpy, i);
-                gdk_window_add_filter (gdk_screen_get_root_window (screen),
+                window = gdk_screen_get_root_window (screen);
+                xwindow = GDK_WINDOW_XID (window);
+
+                gdk_window_add_filter (window,
                                        (GdkFilterFunc) keybindings_filter,
                                        manager);
+
+                gdk_error_trap_push ();
+                /* Add KeyPressMask to the currently reportable event masks */
+                XGetWindowAttributes (xdpy, xwindow, &atts);
+                XSelectInput (xdpy, xwindow, atts.your_event_mask | KeyPressMask);
+                gdk_error_trap_pop_ignored ();
         }
         manager->priv->screens = get_screens_list ();
 
@@ -564,18 +573,9 @@ msd_keybindings_manager_start (MsdKeybindingsManager *manager,
         bindings_get_entries (manager);
         binding_register_keys (manager);
 
-        /* DConf has different API between versions:
-         * http://developer.gnome.org/dconf/0.12/DConfClient.html
-         * http://developer.gnome.org/dconf/0.14/DConfClient.html
-         */
-        #ifdef HAVE_DCONF_0_13
         manager->priv->client = dconf_client_new ();
         dconf_client_watch_fast (manager->priv->client, GSETTINGS_KEYBINDINGS_DIR);
         g_signal_connect (manager->priv->client, "changed", G_CALLBACK (bindings_callback), manager);
-        #else
-        manager->priv->client = dconf_client_new (NULL, (DConfWatchFunc) bindings_callback, manager, NULL);
-        dconf_client_watch (manager->priv->client, GSETTINGS_KEYBINDINGS_DIR, NULL, NULL);
-        #endif
 
         mate_settings_profile_end (NULL);
 
@@ -609,76 +609,11 @@ msd_keybindings_manager_stop (MsdKeybindingsManager *manager)
         p->screens = NULL;
 }
 
-static void
-msd_keybindings_manager_set_property (GObject        *object,
-                               guint           prop_id,
-                               const GValue   *value,
-                               GParamSpec     *pspec)
-{
-        MsdKeybindingsManager *self;
-
-        self = MSD_KEYBINDINGS_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-msd_keybindings_manager_get_property (GObject        *object,
-                               guint           prop_id,
-                               GValue         *value,
-                               GParamSpec     *pspec)
-{
-        MsdKeybindingsManager *self;
-
-        self = MSD_KEYBINDINGS_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static GObject *
-msd_keybindings_manager_constructor (GType                  type,
-                              guint                  n_construct_properties,
-                              GObjectConstructParam *construct_properties)
-{
-        MsdKeybindingsManager      *keybindings_manager;
-        MsdKeybindingsManagerClass *klass;
-
-        klass = MSD_KEYBINDINGS_MANAGER_CLASS (g_type_class_peek (MSD_TYPE_KEYBINDINGS_MANAGER));
-
-        keybindings_manager = MSD_KEYBINDINGS_MANAGER (G_OBJECT_CLASS (msd_keybindings_manager_parent_class)->constructor (type,
-                                                                                                      n_construct_properties,
-                                                                                                      construct_properties));
-
-        return G_OBJECT (keybindings_manager);
-}
-
-static void
-msd_keybindings_manager_dispose (GObject *object)
-{
-        MsdKeybindingsManager *keybindings_manager;
-
-        keybindings_manager = MSD_KEYBINDINGS_MANAGER (object);
-
-        G_OBJECT_CLASS (msd_keybindings_manager_parent_class)->dispose (object);
-}
-
 static void
 msd_keybindings_manager_class_init (MsdKeybindingsManagerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->get_property = msd_keybindings_manager_get_property;
-        object_class->set_property = msd_keybindings_manager_set_property;
-        object_class->constructor = msd_keybindings_manager_constructor;
-        object_class->dispose = msd_keybindings_manager_dispose;
         object_class->finalize = msd_keybindings_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (MsdKeybindingsManagerPrivate));
