@@ -35,15 +35,14 @@
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtk.h>
 
 #ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
-	#include <X11/extensions/xf86misc.h>
+#include <X11/extensions/xf86misc.h>
 #endif
 
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-	#include <X11/XKBlib.h>
-	#include <X11/keysym.h>
+#include <X11/XKBlib.h>
+#include <X11/keysym.h>
 #endif
 
 #include "mate-settings-profile.h"
@@ -304,11 +303,7 @@ apply_settings (GSettings          *settings,
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         XSync (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), FALSE);
-#if GTK_CHECK_VERSION (3, 0, 0)
         gdk_error_trap_pop_ignored ();
-#else
-        gdk_error_trap_pop ();
-#endif
 }
 
 void
