@@ -107,8 +107,7 @@ static void     msd_mouse_manager_class_init  (MsdMouseManagerClass *klass);
 static void     msd_mouse_manager_init        (MsdMouseManager      *mouse_manager);
 static void     msd_mouse_manager_finalize    (GObject             *object);
 static void     set_mouse_settings            (MsdMouseManager      *manager);
-static void     set_tap_to_click              (MsdMouseManager      *manager,
-                                               XDeviceInfo          *device_info,
+static void     set_tap_to_click_synaptics    (XDeviceInfo          *device_info,
                                                gboolean              state,
                                                gboolean              left_handed,
                                                gint                  one_finger_tap,
@@ -244,11 +243,103 @@ touchpad_has_single_button (XDevice *device)
         return is_single_button;
 }
 
+static gboolean
+property_exists_on_device (XDeviceInfo *device_info,
+                           const char  *property_name)
+{
+        XDevice *device;
+        int rc;
+        Atom type, prop;
+        int format;
+        unsigned long nitems, bytes_after;
+        unsigned char *data;
+
+        prop = property_from_name (property_name);
+        if (!prop)
+                return FALSE;
+
+        gdk_error_trap_push ();
+        device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info->id);
+        if ((gdk_error_trap_pop () != 0) || (device == NULL))
+                return FALSE;
+
+        gdk_error_trap_push ();
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                 device, prop, 0, 1, False, XA_INTEGER, &type, &format,
+                                 &nitems, &bytes_after, &data);
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        gdk_error_trap_pop_ignored ();
+
+        return rc == Success;
+}
+
 static void
-set_left_handed (MsdMouseManager *manager,
-                 XDeviceInfo     *device_info,
-                 gboolean         mouse_left_handed,
-                 gboolean         touchpad_left_handed)
+property_set_bool (XDeviceInfo *device_info,
+                   XDevice     *device,
+                   const char  *property_name,
+                   int          property_index,
+                   gboolean     enabled)
+{
+        int rc;
+        unsigned long nitems, bytes_after;
+        unsigned char *data;
+        int act_format;
+        Atom act_type, property;
+
+        property = property_from_name (property_name);
+        if (!property)
+                return;
+
+        gdk_error_trap_push ();
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
+                                 property, 0, 1, False,
+                                 XA_INTEGER, &act_type, &act_format, &nitems,
+                                 &bytes_after, &data);
+
+        if (rc == Success && act_type == XA_INTEGER && act_format == 8 && nitems > property_index) {
+                data[property_index] = enabled ? 1 : 0;
+                XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
+                                       property, XA_INTEGER, 8,
+                                       PropModeReplace, data, nitems);
+        }
+
+        if (rc == Success)
+                XFree (data);
+
+        if (gdk_error_trap_pop ()) {
+                g_warning ("Error while setting %s on \"%s\"", property_name, device_info->name);
+        }
+}
+
+static void
+touchpad_set_bool (XDeviceInfo *device_info,
+                   const char  *property_name,
+                   int          property_index,
+                   gboolean     enabled)
+{
+        XDevice *device;
+
+        device = device_is_touchpad (device_info);
+        if (device == NULL) {
+                return;
+        }
+
+        property_set_bool (device_info, device, property_name, property_index, enabled);
+
+        gdk_error_trap_push ();
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+set_left_handed_legacy_driver (MsdMouseManager *manager,
+                               XDeviceInfo     *device_info,
+                               gboolean         mouse_left_handed,
+                               gboolean         touchpad_left_handed)
 {
         XDevice *device;
         guchar *buttons;
@@ -275,7 +366,7 @@ set_left_handed (MsdMouseManager *manager,
                         gint one_finger_tap = g_settings_get_int (manager->priv->settings_touchpad, KEY_TOUCHPAD_ONE_FINGER_TAP);
                         gint two_finger_tap = g_settings_get_int (manager->priv->settings_touchpad, KEY_TOUCHPAD_TWO_FINGER_TAP);
                         gint three_finger_tap = g_settings_get_int (manager->priv->settings_touchpad, KEY_TOUCHPAD_THREE_FINGER_TAP);
-                        set_tap_to_click (manager, device_info, tap, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
+                        set_tap_to_click_synaptics (device_info, tap, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
                 }
 
                 XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
@@ -317,6 +408,48 @@ set_left_handed (MsdMouseManager *manager,
         g_free (buttons);
 }
 
+static void
+set_left_handed_libinput (XDeviceInfo *device_info,
+                          gboolean     mouse_left_handed,
+                          gboolean     touchpad_left_handed)
+{
+        XDevice *device;
+        gboolean want_lefthanded;
+
+        device = device_is_touchpad (device_info);
+        if (device == NULL) {
+                gdk_error_trap_push ();
+                device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info->id);
+                if ((gdk_error_trap_pop () != 0) || (device == NULL))
+                        return;
+
+                want_lefthanded = mouse_left_handed;
+        } else {
+                /* touchpad device is already open after
+                 * return from device_is_touchpad function
+                 */
+                want_lefthanded = touchpad_left_handed;
+        }
+
+        property_set_bool (device_info, device, "libinput Left Handed Enabled", 0, want_lefthanded);
+
+        gdk_error_trap_push ();
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+set_left_handed (MsdMouseManager *manager,
+                 XDeviceInfo     *device_info,
+                 gboolean         mouse_left_handed,
+                 gboolean         touchpad_left_handed)
+{
+        if (property_exists_on_device (device_info, "libinput Left Handed Enabled"))
+                set_left_handed_libinput (device_info, mouse_left_handed, touchpad_left_handed);
+        else
+                set_left_handed_legacy_driver (manager, device_info, mouse_left_handed, touchpad_left_handed);
+}
+
 static void
 set_left_handed_all (MsdMouseManager *manager,
                      gboolean         mouse_left_handed,
@@ -378,8 +511,8 @@ set_devicepresence_handler (MsdMouseManager *manager)
 }
 
 static void
-set_motion (MsdMouseManager *manager,
-            XDeviceInfo     *device_info)
+set_motion_legacy_driver (MsdMouseManager *manager,
+                          XDeviceInfo     *device_info)
 {
         XDevice *device;
         XPtrFeedbackControl feedback;
@@ -467,6 +600,92 @@ set_motion (MsdMouseManager *manager,
         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
 }
 
+static void
+set_motion_libinput (MsdMouseManager *manager,
+                     XDeviceInfo     *device_info)
+{
+        XDevice *device;
+        Atom prop;
+        Atom type;
+        Atom float_type;
+        int format, rc;
+        unsigned long nitems, bytes_after;
+        GSettings *settings;
+        union {
+                unsigned char *c;
+                long *l;
+        } data;
+        gfloat accel;
+        gfloat motion_acceleration;
+
+        float_type = property_from_name ("FLOAT");
+        if (!float_type)
+                return;
+
+        prop = property_from_name ("libinput Accel Speed");
+        if (!prop) {
+                return;
+        }
+
+        device = device_is_touchpad (device_info);
+        if (device != NULL) {
+                settings = manager->priv->settings_touchpad;
+        } else {
+                gdk_error_trap_push ();
+                device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info->id);
+                if ((gdk_error_trap_pop () != 0) || (device == NULL))
+                        return;
+
+                settings = manager->priv->settings_mouse;
+        }
+
+        /* Calculate acceleration */
+        motion_acceleration = g_settings_get_double (settings, KEY_MOTION_ACCELERATION);
+
+        /* panel gives us a range of 1.0-10.0, map to libinput's [-1, 1]
+         *
+         * oldrange = (oldmax - oldmin)
+         * newrange = (newmax - newmin)
+         *
+         * mapped = (value - oldmin) * newrange / oldrange + oldmin
+         */
+
+        if (motion_acceleration == -1.0) /* unset */
+                accel = 0.0;
+        else
+                accel = (motion_acceleration - 1.0) * 2.0 / 9.0 - 1;
+
+        gdk_error_trap_push ();
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                 device, prop, 0, 1, False, float_type, &type, &format,
+                                 &nitems, &bytes_after, &data.c);
+
+        if (rc == Success && type == float_type && format == 32 && nitems >= 1) {
+                *(float *) data.l = accel;
+                XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                       device, prop, float_type, 32, PropModeReplace, data.c, nitems);
+        }
+
+        if (rc == Success) {
+                XFree (data.c);
+        }
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        if (gdk_error_trap_pop ()) {
+                g_warning ("Error while setting accel speed on \"%s\"", device_info->name);
+        }
+}
+
+static void
+set_motion (MsdMouseManager *manager,
+            XDeviceInfo     *device_info)
+{
+        if (property_exists_on_device (device_info, "libinput Accel Speed"))
+                set_motion_libinput (manager, device_info);
+        else
+                set_motion_legacy_driver (manager, device_info);
+}
+
 static void
 set_motion_all (MsdMouseManager *manager)
 {
@@ -485,9 +704,8 @@ set_motion_all (MsdMouseManager *manager)
 }
 
 static void
-set_middle_button (MsdMouseManager *manager,
-                   XDeviceInfo     *device_info,
-                   gboolean         middle_button)
+set_middle_button_evdev (XDeviceInfo *device_info,
+                         gboolean     middle_button)
 {
         XDevice *device;
         Atom prop;
@@ -529,8 +747,47 @@ set_middle_button (MsdMouseManager *manager,
 }
 
 static void
-set_middle_button_all (MsdMouseManager *manager,
-                       gboolean         middle_button)
+set_middle_button_libinput (XDeviceInfo *device_info,
+                            gboolean     middle_button)
+{
+        XDevice *device;
+
+        /* touchpad devices are excluded as the old code
+         * only applies to evdev devices
+         */
+        device = device_is_touchpad (device_info);
+        if (device != NULL) {
+                gdk_error_trap_push ();
+                XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+                gdk_error_trap_pop_ignored ();
+                return;
+        }
+
+        gdk_error_trap_push ();
+        device = XOpenDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device_info->id);
+        if ((gdk_error_trap_pop () != 0) || (device == NULL))
+                return;
+
+        property_set_bool (device_info, device, "libinput Middle Emulation Enabled", 0, middle_button);
+
+        gdk_error_trap_push ();
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+set_middle_button (XDeviceInfo *device_info,
+                   gboolean     middle_button)
+{
+        if (property_from_name ("Evdev Middle Button Emulation"))
+                set_middle_button_evdev (device_info, middle_button);
+
+        if (property_from_name ("libinput Middle Emulation Enabled"))
+                set_middle_button_libinput (device_info, middle_button);
+}
+
+static void
+set_middle_button_all (gboolean middle_button)
 {
         XDeviceInfo *device_info;
         gint n_devices;
@@ -539,7 +796,7 @@ set_middle_button_all (MsdMouseManager *manager,
         device_info = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &n_devices);
 
         for (i = 0; i < n_devices; i++) {
-                set_middle_button (manager, &device_info[i], middle_button);
+                set_middle_button (&device_info[i], middle_button);
         }
 
         if (device_info != NULL)
@@ -559,9 +816,9 @@ have_program_in_path (const char *name)
 }
 
 static void
-set_disable_w_typing (MsdMouseManager *manager, gboolean state)
+set_disable_w_typing_synaptics (MsdMouseManager *manager,
+                                gboolean         state)
 {
-
         if (state && touchpad_is_present ()) {
                 GError *error = NULL;
                 char *args[6];
@@ -598,6 +855,38 @@ set_disable_w_typing (MsdMouseManager *manager, gboolean state)
         }
 }
 
+static void
+set_disable_w_typing_libinput (MsdMouseManager *manager,
+                               gboolean         state)
+{
+        XDeviceInfo *device_info;
+        gint n_devices;
+        gint i;
+
+        /* This is only called once for synaptics but for libinput
+         * we need to loop through the list of devices
+         */
+        device_info = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &n_devices);
+
+        for (i = 0; i < n_devices; i++) {
+                touchpad_set_bool (&device_info[i], "libinput Disable While Typing Enabled", 0, state);
+        }
+
+        if (device_info != NULL)
+                XFreeDeviceList (device_info);
+}
+
+static void
+set_disable_w_typing (MsdMouseManager *manager,
+                      gboolean         state)
+{
+        if (property_from_name ("Synaptics Off"))
+                set_disable_w_typing_synaptics (manager, state);
+
+        if (property_from_name ("libinput Disable While Typing Enabled"))
+                set_disable_w_typing_libinput (manager, state);
+}
+
 static gboolean
 get_touchpad_handedness (MsdMouseManager *manager,
                          gboolean         mouse_left_handed)
@@ -615,13 +904,12 @@ get_touchpad_handedness (MsdMouseManager *manager,
 }
 
 static void
-set_tap_to_click (MsdMouseManager *manager,
-                  XDeviceInfo     *device_info,
-                  gboolean         state,
-                  gboolean         left_handed,
-                  gint             one_finger_tap,
-                  gint             two_finger_tap,
-                  gint             three_finger_tap)
+set_tap_to_click_synaptics (XDeviceInfo *device_info,
+                            gboolean     state,
+                            gboolean     left_handed,
+                            gint         one_finger_tap,
+                            gint         two_finger_tap,
+                            gint         three_finger_tap)
 {
         XDevice *device;
         int format, rc;
@@ -670,6 +958,29 @@ set_tap_to_click (MsdMouseManager *manager,
         }
 }
 
+static void
+set_tap_to_click_libinput (XDeviceInfo *device_info,
+                           gboolean     state)
+{
+        touchpad_set_bool (device_info, "libinput Tapping Enabled", 0, state);
+}
+
+static void
+set_tap_to_click (XDeviceInfo *device_info,
+                  gboolean     state,
+                  gboolean     left_handed,
+                  gint         one_finger_tap,
+                  gint         two_finger_tap,
+                  gint         three_finger_tap)
+{
+        if (property_from_name ("Synaptics Tap Action"))
+                set_tap_to_click_synaptics (device_info, state, left_handed,
+                                            one_finger_tap, two_finger_tap, three_finger_tap);
+
+        if (property_from_name ("libinput Tapping Enabled"))
+                set_tap_to_click_libinput (device_info, state);
+}
+
 static void
 set_tap_to_click_all (MsdMouseManager *manager)
 {
@@ -686,17 +997,16 @@ set_tap_to_click_all (MsdMouseManager *manager)
         gint three_finger_tap = g_settings_get_int (manager->priv->settings_touchpad, KEY_TOUCHPAD_THREE_FINGER_TAP);
 
         for (i = 0; i < numdevices; i++) {
-                set_tap_to_click (manager, &devicelist[i], state, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
+                set_tap_to_click (&devicelist[i], state, left_handed, one_finger_tap, two_finger_tap, three_finger_tap);
         }
 
         XFreeDeviceList (devicelist);
 }
 
 static void
-set_click_actions (MsdMouseManager *manager,
-                   XDeviceInfo     *device_info,
-                   gint             enable_two_finger_click,
-                   gint             enable_three_finger_click)
+set_click_actions_synaptics (XDeviceInfo *device_info,
+                             gint         enable_two_finger_click,
+                             gint         enable_three_finger_click)
 {
         XDevice *device;
         int format, rc;
@@ -735,7 +1045,66 @@ set_click_actions (MsdMouseManager *manager,
         if (gdk_error_trap_pop ()) {
                 g_warning ("Error in setting click actions on \"%s\"", device_info->name);
         }
+}
 
+static void
+set_click_actions_libinput (XDeviceInfo *device_info,
+                            gint         enable_two_finger_click,
+                            gint         enable_three_finger_click)
+{
+        XDevice *device;
+        int format, rc;
+        unsigned long nitems, bytes_after;
+        unsigned char *data;
+        Atom prop, type;
+        gboolean want_clickfinger;
+        gboolean want_softwarebuttons;
+
+        prop = property_from_name ("libinput Click Method Enabled");
+        if (!prop)
+                return;
+
+        device = device_is_touchpad (device_info);
+        if (device == NULL) {
+                return;
+        }
+
+        g_debug ("setting click action to click on %s", device_info->name);
+
+        want_clickfinger = enable_two_finger_click || enable_three_finger_click;
+        want_softwarebuttons = !want_clickfinger;
+
+        gdk_error_trap_push ();
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device, prop, 0, 2,
+                                 False, XA_INTEGER, &type, &format, &nitems,
+                                 &bytes_after, &data);
+
+        if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 2) {
+                data[0] = want_softwarebuttons;
+                data[1] = want_clickfinger;
+                XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device, prop,
+                                       XA_INTEGER, 8, PropModeReplace, data, nitems);
+        }
+
+        if (rc == Success)
+                XFree (data);
+
+        XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
+        if (gdk_error_trap_pop ()) {
+                g_warning ("Error in setting click actions on \"%s\"", device_info->name);
+        }
+}
+
+static void
+set_click_actions (XDeviceInfo *device_info,
+                   gint         enable_two_finger_click,
+                   gint         enable_three_finger_click)
+{
+        if (property_from_name ("Synaptics Click Action"))
+                set_click_actions_synaptics (device_info, enable_two_finger_click, enable_three_finger_click);
+
+        if (property_from_name ("libinput Click Method Enabled"))
+                set_click_actions_libinput (device_info, enable_two_finger_click, enable_three_finger_click);
 }
 
 static void
@@ -751,16 +1120,15 @@ set_click_actions_all (MsdMouseManager *manager)
         gint enable_three_finger_click = g_settings_get_int (manager->priv->settings_touchpad, KEY_TOUCHPAD_THREE_FINGER_CLICK);
 
         for (i = 0; i < numdevices; i++) {
-                set_click_actions (manager, &devicelist[i], enable_two_finger_click, enable_three_finger_click);
+                set_click_actions (&devicelist[i], enable_two_finger_click, enable_three_finger_click);
         }
 
         XFreeDeviceList (devicelist);
 }
 
 static void
-set_natural_scroll (MsdMouseManager *manager,
-                    XDeviceInfo     *device_info,
-                    gboolean         natural_scroll)
+set_natural_scroll_synaptics (XDeviceInfo *device_info,
+                              gboolean     natural_scroll)
 {
         XDevice *device;
         int format, rc;
@@ -808,6 +1176,25 @@ set_natural_scroll (MsdMouseManager *manager,
         }
 }
 
+static void
+set_natural_scroll_libinput (XDeviceInfo *device_info,
+                             gboolean     natural_scroll)
+{
+        g_debug ("Trying to set %s for \"%s\"", natural_scroll ? "natural (reverse) scroll" : "normal scroll", device_info->name);
+
+        touchpad_set_bool (device_info, "libinput Natural Scrolling Enabled", 0, natural_scroll);
+}
+
+static void
+set_natural_scroll (XDeviceInfo *device_info,
+                    gboolean     natural_scroll)
+{
+        if (property_from_name ("Synaptics Scrolling Distance"))
+                set_natural_scroll_synaptics (device_info, natural_scroll);
+
+        if (property_from_name ("libinput Natural Scrolling Enabled"))
+                set_natural_scroll_libinput (device_info, natural_scroll);
+}
 
 static void
 set_natural_scroll_all (MsdMouseManager *manager)
@@ -821,27 +1208,36 @@ set_natural_scroll_all (MsdMouseManager *manager)
         gboolean natural_scroll = g_settings_get_boolean (manager->priv->settings_touchpad, KEY_TOUCHPAD_NATURAL_SCROLL);
 
         for (i = 0; i < numdevices; i++) {
-                set_natural_scroll (manager, &devicelist[i], natural_scroll);
+                set_natural_scroll (&devicelist[i], natural_scroll);
         }
 
         XFreeDeviceList (devicelist);
 }
 
 static void
-synaptics_set_bool (XDeviceInfo *device_info,
-                    const char  *property_name,
-                    int          property_index,
-                    gboolean     enabled)
+set_scrolling_synaptics (XDeviceInfo *device_info,
+                         GSettings   *settings)
+{
+        touchpad_set_bool (device_info, "Synaptics Edge Scrolling", 0, g_settings_get_boolean (settings, KEY_VERT_EDGE_SCROLL));
+        touchpad_set_bool (device_info, "Synaptics Edge Scrolling", 1, g_settings_get_boolean (settings, KEY_HORIZ_EDGE_SCROLL));
+        touchpad_set_bool (device_info, "Synaptics Two-Finger Scrolling", 0, g_settings_get_boolean (settings, KEY_VERT_TWO_FINGER_SCROLL));
+        touchpad_set_bool (device_info, "Synaptics Two-Finger Scrolling", 1, g_settings_get_boolean (settings, KEY_HORIZ_TWO_FINGER_SCROLL));
+}
+
+static void
+set_scrolling_libinput (XDeviceInfo *device_info,
+                        GSettings   *settings)
 {
         XDevice *device;
-        int rc;
+        int format, rc;
         unsigned long nitems, bytes_after;
         unsigned char *data;
-        int act_format;
-        Atom act_type, property;
+        Atom prop, type;
+        gboolean want_edge, want_2fg;
+        gboolean want_horiz;
 
-        property = property_from_name (property_name);
-        if (!property)
+        prop = property_from_name ("libinput Scroll Method Enabled");
+        if (!prop)
                 return;
 
         device = device_is_touchpad (device_info);
@@ -849,17 +1245,27 @@ synaptics_set_bool (XDeviceInfo *device_info,
                 return;
         }
 
+        want_2fg = g_settings_get_boolean (settings, KEY_VERT_TWO_FINGER_SCROLL);
+        want_edge = g_settings_get_boolean (settings, KEY_VERT_EDGE_SCROLL);
+
+        /* libinput only allows for one scroll method at a time.
+         * If both are set, pick 2fg scrolling.
+         */
+        if (want_2fg)
+                want_edge = FALSE;
+
+        g_debug ("setting scroll method on %s", device_info->name);
+
         gdk_error_trap_push ();
-        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
-                                 property, 0, 1, False,
-                                 XA_INTEGER, &act_type, &act_format, &nitems,
+        rc = XGetDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device, prop, 0, 2,
+                                 False, XA_INTEGER, &type, &format, &nitems,
                                  &bytes_after, &data);
 
-        if (rc == Success && act_type == XA_INTEGER && act_format == 8 && nitems > property_index) {
-                data[property_index] = enabled ? 1 : 0;
+        if (rc == Success && type == XA_INTEGER && format == 8 && nitems >= 3) {
+                data[0] = want_2fg;
+                data[1] = want_edge;
                 XChangeDeviceProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device,
-                                       property, XA_INTEGER, 8,
-                                       PropModeReplace, data, nitems);
+                                       prop, XA_INTEGER, 8, PropModeReplace, data, nitems);
         }
 
         if (rc == Success)
@@ -867,14 +1273,36 @@ synaptics_set_bool (XDeviceInfo *device_info,
 
         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
         if (gdk_error_trap_pop ()) {
-                g_warning ("Error while setting %s on \"%s\"", property_name, device_info->name);
+                g_warning ("Error in setting scroll method on \"%s\"", device_info->name);
         }
+
+        /* Horizontal scrolling is handled by xf86-input-libinput and
+         * there's only one bool. Pick the one matching the scroll method
+         * we picked above.
+         */
+        if (want_2fg)
+                want_horiz = g_settings_get_boolean (settings, KEY_HORIZ_TWO_FINGER_SCROLL);
+        else if (want_edge)
+                want_horiz = g_settings_get_boolean (settings, KEY_HORIZ_EDGE_SCROLL);
+        else
+                return;
+
+        touchpad_set_bool (device_info, "libinput Horizontal Scroll Enabled", 0, want_horiz);
 }
 
 static void
-synaptics_set_bool_all (const char *property_name,
-                        int         property_index,
-                        gboolean    enabled)
+set_scrolling (XDeviceInfo *device_info,
+               GSettings   *settings)
+{
+        if (property_from_name ("Synaptics Edge Scrolling"))
+                set_scrolling_synaptics (device_info, settings);
+
+        if (property_from_name ("libinput Scroll Method Enabled"))
+                set_scrolling_libinput (device_info, settings);
+}
+
+static void
+set_scrolling_all (GSettings *settings)
 {
         int numdevices, i;
         XDeviceInfo *devicelist = XListInputDevices (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &numdevices);
@@ -883,21 +1311,12 @@ synaptics_set_bool_all (const char *property_name,
                 return;
 
         for (i = 0; i < numdevices; i++) {
-                synaptics_set_bool (&devicelist[i], property_name, property_index, enabled);
+                set_scrolling (&devicelist[i], settings);
         }
 
         XFreeDeviceList (devicelist);
 }
 
-static void
-set_scrolling (GSettings *settings)
-{
-        synaptics_set_bool_all ("Synaptics Edge Scrolling", 0, g_settings_get_boolean (settings, KEY_VERT_EDGE_SCROLL));
-        synaptics_set_bool_all ("Synaptics Edge Scrolling", 1, g_settings_get_boolean (settings, KEY_HORIZ_EDGE_SCROLL));
-        synaptics_set_bool_all ("Synaptics Two-Finger Scrolling", 0, g_settings_get_boolean (settings, KEY_VERT_TWO_FINGER_SCROLL));
-        synaptics_set_bool_all ("Synaptics Two-Finger Scrolling", 1, g_settings_get_boolean (settings, KEY_HORIZ_TWO_FINGER_SCROLL));
-}
-
 static void
 set_touchpad_enabled (XDeviceInfo *device_info,
                       gboolean     state)
@@ -1043,13 +1462,13 @@ set_mouse_settings (MsdMouseManager *manager)
         set_left_handed_all (manager, mouse_left_handed, touchpad_left_handed);
 
         set_motion_all (manager);
-        set_middle_button_all (manager, g_settings_get_boolean (manager->priv->settings_mouse, KEY_MIDDLE_BUTTON_EMULATION));
+        set_middle_button_all (g_settings_get_boolean (manager->priv->settings_mouse, KEY_MIDDLE_BUTTON_EMULATION));
 
         set_disable_w_typing (manager, g_settings_get_boolean (manager->priv->settings_touchpad, KEY_TOUCHPAD_DISABLE_W_TYPING));
 
         set_tap_to_click_all (manager);
         set_click_actions_all (manager);
-        set_scrolling (manager->priv->settings_touchpad);
+        set_scrolling_all (manager->priv->settings_touchpad);
         set_natural_scroll_all (manager);
         set_touchpad_enabled_all (g_settings_get_boolean (manager->priv->settings_touchpad, KEY_TOUCHPAD_ENABLED));
 }
@@ -1067,7 +1486,7 @@ mouse_callback (GSettings          *settings,
                 || (g_strcmp0 (key, KEY_MOTION_THRESHOLD) == 0)) {
                 set_motion_all (manager);
         } else if (g_strcmp0 (key, KEY_MIDDLE_BUTTON_EMULATION) == 0) {
-                set_middle_button_all (manager, g_settings_get_boolean (settings, key));
+                set_middle_button_all (g_settings_get_boolean (settings, key));
         } else if (g_strcmp0 (key, KEY_MOUSE_LOCATE_POINTER) == 0) {
                 set_locate_pointer (manager, g_settings_get_boolean (settings, key));
 #if 0   /* FIXME need to fork (?) mousetweaks for this to work */
@@ -1107,7 +1526,7 @@ touchpad_callback (GSettings          *settings,
                 || (g_strcmp0 (key, KEY_HORIZ_EDGE_SCROLL) == 0)
                 || (g_strcmp0 (key, KEY_VERT_TWO_FINGER_SCROLL) == 0)
                 || (g_strcmp0 (key, KEY_HORIZ_TWO_FINGER_SCROLL) == 0)) {
-                set_scrolling (manager->priv->settings_touchpad);
+                set_scrolling_all (manager->priv->settings_touchpad);
         } else if (g_strcmp0 (key, KEY_TOUCHPAD_NATURAL_SCROLL) == 0) {
                 set_natural_scroll_all (manager);
         } else if (g_strcmp0 (key, KEY_TOUCHPAD_ENABLED) == 0) {
